%% --------------------------------------------------------------------------------------------------
% invert_glm_mcmc
%
%---------------------------------------------------------------------------------------------
% INPUT:
%       data
%       params
%       nIterations
%       nBurnin
%
% Optional:
%
%--------------------------------------------------------------------------------------------
% OUTPUT: 
%       result
%
% -------------------------------------------------------------------------------------------
% REFERENCE:
%
% Author:
%     Sudhir Shankar Raman , Nov 2015 
%

%%
function [result] = invert_glm_mcmc(data,params,nIterations,nBurnin)

% initialize
x= data.x ;
y= data.y ;
[N,D] = size(x);

trace.param.beta = randn(D,1);
trace.param.epsilon = zeros(nIterations,1);
trace.llk = zeros(nIterations,1);
trace.llkj = zeros(nIterations,1);

% choose a random starting point
trace.current.beta = 1*randn(D,1);

% Fixed for this example
trace.current.epsilon = 10;% gamrnd(1,1);
llk = -0.5*params.T*trace.current.epsilon*sum((y-x*trace.current.beta).^2);
llp = -0.5*params.prior.tau*sum((trace.current.beta - params.prior.mu).^2);
trace.current.logjoint = llk + llp;
trace.current.loglk = llk ;

% parameters for adapting the proposal distribution
for iParam=1:D
    proposal.params(iParam).mu = trace.current.beta(iParam);
    proposal.params(iParam).Sigma = 1;%*eye(D);
    proposal.params(iParam).lambda = 2.4^2;%/D;
    proposal.params(iParam).nAccept = 0;
end
proposal.tune1 = round(nIterations/3);
proposal.factor1 = -0.7;
proposal.ratio_target = 0.234;

%---------------------- The main MCMC steps ------------------------%
for i=1:nIterations

    proposal.iteration = i;
    
    % sample each beta value keeping others fixed using Metropolis Hastings
    for iParam = 1:D   
        newbeta = trace.current.beta;
        newbeta(iParam) = mvnrnd(trace.current.beta(iParam),proposal.params(iParam).lambda*proposal.params(iParam).Sigma)';
        llk_new = -0.5*params.T*trace.current.epsilon*sum((y-x*newbeta).^2);
        llp_new = -0.5*params.prior.tau*sum((newbeta - params.prior.mu).^2);
        logjoint_new = llk_new + llp_new;

        % acceptance check
        if(exp(logjoint_new - trace.current.logjoint) > rand())
            %accept
            trace.current.logjoint = logjoint_new;
            trace.current.loglk = llk_new; 
            trace.current.beta = newbeta;
            proposal.params(iParam).nAccept = proposal.params(iParam).nAccept+1;
        end
        trace.param.beta(:,i) = trace.current.beta;
        
        % adapting the proposal distribution
        if(i<nBurnin)
            proposal = adapt_proposal(proposal,trace.current.beta(iParam),iParam);
        end
    end
    
    % Note the logjoint probability and loglikelihood.
    trace.llkj(i) = trace.current.logjoint;        
    trace.llk(i) = -0.5*trace.current.epsilon*sum((y-x*trace.current.beta).^2) ...
                - 0.5*N*log(2*pi) + 0.5*N*log(trace.current.epsilon);        
    
end
trace.proposal =proposal;
result = trace;
result.T = params.T;
result.nBurnin = nBurnin;
result.nIterations = nIterations;
result = estimate_moments(result);

function [result] = estimate_moments(result)

% estimate posterior mean
result.pE = mean(result.param.beta(:,result.nBurnin:end),2);
result.pC = cov(result.param.beta(:,result.nBurnin:end)');

% estimate posterior covariance matrix

% Adapt the proposal distribution for the respective parameter
function [proposal] = adapt_proposal(proposal,currVal,iParam)

if((proposal.iteration < proposal.tune1))
    stepgamma = (proposal.iteration+1)^(proposal.factor1);
    dParam= currVal-proposal.params(iParam).mu;
    proposal.params(iParam).mu = proposal.params(iParam).mu + stepgamma*(dParam);
    proposal.params(iParam).Sigma = proposal.params(iParam).Sigma + ...
            stepgamma*(dParam*dParam' - proposal.params(iParam).Sigma) + 0.0001*eye(length(currVal));
    proposal.params(iParam).lambda = exp(log(proposal.params(iParam).lambda) + ...
            stepgamma*((proposal.params(iParam).nAccept/proposal.iteration)...
            - proposal.ratio_target));
end;